# -*- coding: utf-8 -*-
# Copyright (C) 2020  Nexedi SA and Contributors.
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.

# verify general functionality

import sys
import re

import pytest

from nxdtest import main


@pytest.fixture
def run_nxdtest(tmpdir):
    """Fixture which returns a function which invokes nxdtest in a temporary
    directory, with the provided .nxdtest file content and with arguments
    passed as `argv`.
    """

    def _run_nxdtest(nxdtest_file_content, argv=("nxdtest",)):
        with tmpdir.as_cwd():
            with open(".nxdtest", "w") as f:
                f.write(nxdtest_file_content)
            sys_argv = sys.argv
            sys.argv = argv
            try:
                main()
            finally:
                sys.argv = sys_argv

    return _run_nxdtest


def test_main(run_nxdtest, capsys):
    run_nxdtest(
        """\
TestCase('TESTNAME', ['echo', 'TEST OUPUT'])
"""
    )

    captured = capsys.readouterr()
    output_lines = captured.out.splitlines()
    assert ">>> TESTNAME" in output_lines
    assert "$ echo TEST OUPUT" in output_lines
    assert "TEST OUPUT" in output_lines
    assert re.match("ok\tTESTNAME\t.*s\t# 1t 0e 0f 0s", output_lines[-1])


def test_error_invoking_command(run_nxdtest, capsys):
    run_nxdtest(
        """\
TestCase('TESTNAME', ['not exist command'])
"""
    )

    captured = capsys.readouterr()
    assert "No such file or directory" in captured.err


def test_error_invoking_summary(run_nxdtest, capsys):
    run_nxdtest(
        """\
TestCase('TESTNAME', ['echo'], summaryf="error")
"""
    )

    captured = capsys.readouterr()
    assert "TypeError" in captured.err


def test_run_argument(run_nxdtest, capsys):
    run_nxdtest(
        """\
TestCase('TEST1', ['echo', 'TEST1'])
TestCase('TEST2', ['echo', 'TEST2'])
""",
        argv=["nxdtest", "--run", "TEST1"],
    )

    captured = capsys.readouterr()
    assert "TEST1" in captured.out
    assert "TEST2" not in captured.out
